__doc__ = """
Python wrapper for Loyverse API

Copyright: (c) 2020, Matteo Berchier
"""

__title__ = "loyverse"
__version__ = "0.0.1"
__author__ = "Matteo Berchier <maberchier@gmail.com>"
__copyright__ = "(c) 2020, Matteo Berchier"

from loyverse.client import Client

__all__ = ('Client',)



"""
The Client class encapsulates the endpoints of the Loyverse API and provides a single point of initialization for
API requests.

The Client class exposes the following end-points

* receipts
* customers
"""

from loyverse.api import Api


class Client:
    """Loyverse API client

    Args:
        access_token (str): Access token string to be used to initialize the client
    """

    def __init__(self, access_token: str = None):

        self._api = Api(access_token=access_token)

        self._categories = None
        self._customers = None
        self._discounts = None
        self._employees = None
        self._inventory = None
        self._items = None
        self._merchants = None
        self._modifiers = None
        self._payments = None
        self._pos_devices = None
        self._receipts = None
        self._shifts = None
        self._stores = None
        self._suppliers = None
        self._taxes = None
        self._variants = None

    def request(self, method: str, path: str, params: dict = None):
        """
        Client general request
        """

        return self._api.request(method, path, params=params)

    @property
    def customers(self):
        """
        Customers endpoint

        Returns:
            customers (loyverse.endpoints.Customer): Customer endpoint wrapper
        """

        if self._customers is None:
            from loyverse.endpoints import Customers
            self._customers = Customers(self._api)
        return self._customers

    @property
    def receipts(self):
        """
        Receipts endpoint

        Returns:
            receipts (loyverse.endpoints.Receipts): Receipts endpoint wrapper
        """

        if self._receipts is None:
            from loyverse.endpoints import Receipts
            self._receipts = Receipts(self._api)

        return self._receipts



"""
Base class sharing common properties and methods that can be reused for all endpoints.
The root url for all Loyverse API requests is https://api.loyverse.com/v1.0.
"""

# TODO: Implement throttling of requests to stay below limits per account (300 requests in 300sec)

import os
import requests
from loyverse.exceptions import AccessTokenMissingError


class Api:
    """
    Base API properties for all endpoints
    """

    def __init__(self, access_token: str = None):
        """
        Api initialization

        Args:
            access_token (str): API access token, can also be defined in the environment variable LOYVERSE_ACCESS_TOKEN
        Notes:
            Initializes the hostname, version and name, as well as the headers containing the access token
        """

        self.name = 'loyverse'
        self.version = '1.0'
        self.url = 'https://api.loyverse.com/v1.0'

        if access_token is None:
            try:
                self._access_token = os.getenv('LOYVERSE_ACCESS_TOKEN')
            except AccessTokenMissingError:
                raise AccessTokenMissingError('LOYVERSE_ACCESS_TOKEN not found in environment variables.')
        else:
            self._access_token = access_token

        self._header = {
            'Authorization': f'Bearer {self._access_token}'
        }

    def request(self, method: str, path: str, params: dict = None) -> dict:
        """
        API request method

        Args:
            method (str): HTTP method (GET, POST, PUT, PATCH, DELETE)
            path (str): API resource path
            params (dict): query parameters dictionary for passed-in path

        Returns:
            response (dict): parsed JSON response
        """

        if path == '' or path is None:
            url = self.url
        else:
            url = f'{self.url}/{path}'

        if method.lower() == 'get':
            response = self.get_request(url, params)
        else:
            # TODO: Implement PUT, PATCH, DELETE, POST methods
            response = None

        return response

    def get_request(self, url: str, params: dict) -> dict:
        """
        GET method for API requests

        Args:
            url (str): complete url (host + path) for the request
            params (dict): query parameters
        Returns:
            response (dict): parse JSON response
        Notes:
            Function maximizes query limit to maximum available (250) and merges multiple responses if response
            size is larger than maximum (250)
        """

        limit_max = 250
        if params is not None:
            params['limit'] = limit_max

        cursor = True
        response_full = dict()

        while cursor:
            response = requests.get(url, headers=self._header, params=params)
            response.raise_for_status()
            response = response.json()

            if 'cursor' in response:
                params = {
                    'cursor': response['cursor'],
                    'limit': limit_max,
                }
            else:
                cursor = False

            key = list(response.keys())[0]
            if key in response_full:
                response_full[key].extend(response[key])
            else:
                for key in response.keys():
                    response_full[key] = response[key]

        return response_full



from os import path
from setuptools import setup

# Project
NAME = 'loyverse'
VERSION = '0.1.0'

# Maintainer
MAINTAINER = 'Matteo Berchier'
MAINTAINER_EMAIL = 'maberchier@gmail.com'

# Long description
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'PYPI.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

# License
LICENSE = 'MIT'

# Project URLs
REPOSITORY = 'https://github.com/matteobe/loyverse'
HOMEPAGE = 'https://loyverse.readthedocs.io'
PROJECT_URLS = {
    'Bug Tracker': f'{REPOSITORY}/issues',
    'Documentation': 'https://loyverse.readthedocs.io',
    'Source Code': REPOSITORY,
}
DOWNLOAD_URL = ''

# Classifiers
CLASSIFIERS = [
    "Development Status :: 2 - Pre-Alpha",
    "Environment :: Console",
    "Operating System :: OS Independent",
    "Intended Audience:: Developers",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.7",
]

# Package definition
setup(name=NAME,
      version=VERSION,
      description='Loyverse API wrapper',
      url=HOMEPAGE,
      packages=[
          'loyverse',
      ],
      maintainer=MAINTAINER,
      maintainer_email=MAINTAINER_EMAIL,
      license=LICENSE,
      long_description=LONG_DESCRIPTION,
      long_description_content_type='text/markdown',
      download_url=DOWNLOAD_URL,
      project_urls=PROJECT_URLS,
      python_requires='>3.7.0',
      install_requires=[
          'requests',
          'python-dotenv',
          'pandas',
      ],
      include_package_data=True,
      zip_safe=False,
      )
